@@ -3,11 +3,12 @@ from .request import StationRESTServerThread, DEFAULT_REST_KWARGS
 from .utils import ProtocolContextLoggingHandler, LocalWebServerLogger
 from .lights import Button, BlinkingLightHTTP, BlinkingLight
 from opentrons.protocol_api import ProtocolContext
+from opentrons.protocol_api.paired_instrument_context import PairedInstrumentContext
 from opentrons.types import Point
 import opentrons.commands.types
 from abc import ABCMeta, abstractmethod
 from functools import wraps, partialmethod
-from itertools import chain
+from itertools import chain, dropwhile
 from opentrons.types import Location
 from typing import Optional, Callable, Tuple
 import json
@@ -70,6 +71,7 @@ class Station(metaclass=StationMeta):
     _protocol_description = "[BRIEFLY DESCRIBE YOUR PROTOCOL]"
     
     def __init__(self,
+        debug_mode = False,
         drop_height: float = -2,
         drop_loc_l: float = 0,
         drop_loc_r: float = 0,
@@ -78,6 +80,8 @@ class Station(metaclass=StationMeta):
         dummy_lights: bool = True,
         jupyter: bool = True,
         log_filepath: Optional[str] = '/var/lib/jupyter/notebooks/outputs/run_{}.log',
+        log_file_format: Optional[str] = '%(asctime)s %(levelname)s %(message)s',
+        log_file_date_format: Optional[str] = '%Y-%m-%d %H:%M:%S',
         log_lws_enable: Optional[str] = True,
         log_lws_ip: Optional[str] = None,
         log_lws_endpoint: str = ":5002/log",
@@ -97,6 +101,7 @@ class Station(metaclass=StationMeta):
         wait_first_log: bool = False,
         **kwargs,
     ):
+        self._debug_mode = debug_mode
         self._drop_height = drop_height
         self._drop_loc_l = drop_loc_l
         self._drop_loc_r = drop_loc_r
@@ -106,6 +111,8 @@ class Station(metaclass=StationMeta):
         self.jupyter = jupyter
         self._language = language
         self._log_filepath = log_filepath.format(time.strftime("%Y_%m_%d__%H_%M_%S"))
+        self._log_file_format = log_file_format
+        self._log_file_date_format = log_file_date_format
         self._log_lws_enable = log_lws_enable
         self._log_lws_ip = log_lws_ip
         self._log_lws_endpoint = log_lws_endpoint
@@ -177,7 +184,9 @@ class Station(metaclass=StationMeta):
         stack_logger.setLevel(self.logger.getEffectiveLevel())
         if self._log_filepath and (self._simulation_log_file or not self._ctx.is_simulating()):
             os.makedirs(os.path.dirname(self._log_filepath), exist_ok=True)
-            stack_logger.addHandler(logging.FileHandler(self._log_filepath))
+            file_handler = logging.FileHandler(self._log_filepath)
+            file_handler.setFormatter(logging.Formatter(self._log_file_format, self._log_file_date_format))
+            stack_logger.addHandler(file_handler)
 
         if self._log_lws_enable:
             self._lws_logger = LocalWebServerLogger(self._log_lws_ip, self._log_lws_endpoint)
@@ -233,22 +242,38 @@ class Station(metaclass=StationMeta):
     def _tipracks(self) -> dict:
         """Mapping from tipracks attribute names to associated pipette (attribute names)"""
         pass
-    
+
+    def _update_tip_log_count(self):
+        """Count the number of used tips"""
+        self._tip_log['count'] = {t: sum(map(lambda x: not x.has_tip, self._tip_log['tips'][t])) for t in self._tipracks().keys()}
+
+    def _reset_tips(self):
+        for key, tips in self._tip_log['tips'].items():
+            for t in tips:
+                t.has_tip = True
+
     def setup_tip_log(self):
-        data = {}
+        data_tip_status = {}
         if self._tip_track:
             self.logger.info(self.msg_format("tip info log", self._tip_log_filepath))
             if os.path.isfile(self._tip_log_filepath):
                 with open(self._tip_log_filepath) as json_file:
-                    data: dict = json.load(json_file).get("count", {})
+                    file = json.load(json_file)
+                    data_tip_status: dict = file.get("tip_status", {})
         else:
             self.logger.debug("not using tip log file")
         
         self._tip_log = {
-            'count': {t: data.get(t, 0) for t in self._tipracks().keys()},
-            'tips': {t: list(chain.from_iterable(map(first_row if getattr(self, p).channels > 1 else wells, getattr(self, t)))) for t, p in self._tipracks().items()},
+            'tips': {t: list(chain.from_iterable(map(first_row if getattr(self, p).channels > 1 else wells, getattr(self, t)))) for t, p in self._tipracks().items()}
         }
+        if data_tip_status:
+            for tiprack, tips in data_tip_status.items():
+                assert len(tips) == len(self._tip_log['tips'][tiprack]), "Wrong length, please reset tip log"
+                for i, tip in enumerate(tips):
+                    assert tip.get("name", "") == str(self._tip_log['tips'][tiprack][i]), "Wrong tip order, please reset tip log"
+                    self._tip_log['tips'][tiprack][i].has_tip = tip.get("has_tip", False)
         self._tip_log['max'] = {t: len(p) for t, p in self._tip_log['tips'].items()}
+        self._update_tip_log_count()
     
     def track_tip(self):
         if self._tip_track and not self._ctx.is_simulating():
@@ -257,36 +282,57 @@ class Station(metaclass=StationMeta):
             with open(self._tip_log_filepath, 'w') as outfile:
                 json.dump({
                     "count": self._tip_log['count'],
-                    "next": {k: str(self._tip_log['tips'][k][v % self._tip_log['max'][k]]) for k, v in self._tip_log['count'].items()},
+                    "next": {k: str(self._get_next_tip(v)) for k, v in self._tip_log['tips'].items()},
+                    "tip_status": {k: [{'name': str(w), 'has_tip': w.has_tip} for w in self._tip_log['tips'][k]] for k, v in self._tip_log['count'].items()}
                 }, outfile, indent=2)
-    
+
+    @staticmethod
+    def _get_next_tip(tips):
+        available_tips = list(dropwhile(lambda x: not x.has_tip, tips))
+        return available_tips[0] if len(available_tips) else None
+
     def pick_up(self, pip, loc: Optional[Location] = None, tiprack: Optional[str] = None):
         if loc is None:
             if tiprack is None:
-                for t in self._tipracks().keys():
-                    if getattr(self, t) == pip.tip_racks:
-                        tiprack = t
-                        break
+                    for t in self._tipracks().keys():
+                        if getattr(self, t) == pip.tip_racks:
+                            tiprack = t
+                            break
             if tiprack is None:
-                raise RuntimeError("no tiprack associated to pipette")
-            
+                if isinstance(pip, PairedInstrumentContext):
+                    # Paired pipette tiprack doesn't work with the for loop below
+                    # pipettes must be the same, so selecting the first tiprack
+                    tiprack = list(self._tipracks().keys())[0]
+                    self._logger.debug("Selectiong {} tiprack for paired pipette".format(tiprack))
+                else:
+                    raise RuntimeError("no tiprack associated to pipette {}".format(pip))
+            self._update_tip_log_count()
             if self._tip_log['count'][tiprack] == self._tip_log['max'][tiprack]:
                 # If empty, wait for refill
-                self._tip_log['count'][tiprack] = 0
+                self._reset_tips()
+                self._update_tip_log_count()
                 self.track_tip()
                 self.pause(self.get_msg_format("refill tips", "\n".join(map(str, getattr(self, tiprack)))))
-            self._tip_log['count'][tiprack] += 1
             self.track_tip()
-            pip.pick_up_tip(self._tip_log['tips'][tiprack][self._tip_log['count'][tiprack] - 1])
-        else:
-            pip.pick_up_tip(loc)
-    
+            loc = self._get_next_tip(self._tip_log['tips'][tiprack])
+        pip.pick_up_tip(loc)
+        self._update_tip_log_count()
+        self.track_tip()
+
+    @staticmethod
+    def _get_pipette_num_channels(pip):
+        pips = pip._instruments.values() if isinstance(pip, PairedInstrumentContext) else [pip]
+        return sum([p.channels for p in pips])
+
     def drop(self, pip):
         # Drop in the Fixed Trash (on 12) at different positions to avoid making a tall heap of tips
         drop_loc = self._ctx.loaded_labwares[12].wells()[0].top(self._drop_height).move(Point(x=self._drop_loc_r if self._side_switch else self._drop_loc_l, y=self._drop_loc_y))
         self._side_switch = not self._side_switch
-        pip.drop_tip(drop_loc)
-        self._drop_count += pip.channels
+        if self._debug_mode:
+            pip.return_tip()
+        else:
+            pip.drop_tip(drop_loc)
+        self._drop_count += self._get_pipette_num_channels(pip)
         if self._drop_count >= self._drop_threshold:
             self.pause('empty tips')
             self._drop_count = 0
@@ -376,6 +422,9 @@ class Station(metaclass=StationMeta):
         self.setup_tip_log()
         self._button.color = 'white'
         self.msg = ""
+
+        if self._debug_mode:
+            self.dual_pause("debug mode", home=(False, False))
         
         try:
             self.body()
